package com.ekexiu.console.system.service;

import com.ekexiu.console.service.Upload;
import com.ekexiu.console.system.dao.ImageDao;
import com.ekexiu.console.system.po.Image;
import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.util.JpgUtil;
import org.jfw.util.StringUtil;
import org.jfw.util.exception.JfwBaseException;
import org.jfw.util.io.IoUtil;

import java.io.*;
import java.sql.Connection;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * Created by TT on 2017/8/24.
 */
@Path("/image")
public class ImageService extends Upload {
    private File imagePath;
    private File tmpPath;
    private File orgPath;
    private File bannerPath;
    private File researchResultPath;
    private String dateFormat = "yyyyMMdd";
    private String dateFormatRes = "yyyyMM";
    private static final String JPG = "jpg";
    private int largeHeadPhotoWidth = 200;
    private int largeHeadPhotoHeight = 200;
    private int middleHeadPhotoWidth = 200;
    private int middleHeadPhotoHeight = 200;
    private int smallHeadPhotoWidth = 200;
    private int smallHeadPhotoHeight = 200;

    private File defaultHeadPhoto;
    private File defaultOrgLogo;
    private File defaultResourcePhoto;

    private byte[] dli;
    private byte[] dmi;
    private byte[] dsi;
    private byte[] dol;
    private byte[] dsl;

    private int resImageMaxLen = 240;
    private File resourceDataPath;

    @Autowrie
    private ImageDao imageDao;

    public File getDefaultHeadPhoto() {
        return defaultHeadPhoto;
    }

    public void setDefaultHeadPhoto(File defaultHeadPhoto) {
        this.defaultHeadPhoto = defaultHeadPhoto;
    }

    public File getDefaultOrgLogo() {
        return defaultOrgLogo;
    }

    public void setDefaultOrgLogo(File defaultOrgLogo) {
        this.defaultOrgLogo = defaultOrgLogo;
    }

    public File getDefaultResourcePhoto() {
        return defaultResourcePhoto;
    }

    public void setDefaultResourcePhoto(File defaultResourcePhoto) {
        this.defaultResourcePhoto = defaultResourcePhoto;
    }

    public File getImagePath() {
        return imagePath;
    }

    public void setImagePath(File imagePath) {
        this.imagePath = imagePath;
        this.orgPath = new File(this.imagePath, "org");
        this.bannerPath = new File(this.imagePath, "banner");
    }

    public File getTmpPath() {
        return tmpPath;
    }

    public void setTmpPath(File tmpPath) {
        this.tmpPath = tmpPath;
    }

    public File getOrgPath() {
        return orgPath;
    }

    public void setOrgPath(File orgPath) {
        this.orgPath = orgPath;
    }

    public File getResearchResultPath() {
        return researchResultPath;
    }

    public void setResearchResultPath(File researchResultPath) {
        this.researchResultPath = researchResultPath;
    }

    public ImageDao getImageDao() {
        return imageDao;
    }

    public void setImageDao(ImageDao imageDao) {
        this.imageDao = imageDao;
    }

    public int getResImageMaxLen() {
        return resImageMaxLen;
    }

    public void setResImageMaxLen(int resImageMaxLen) {
        this.resImageMaxLen = resImageMaxLen;
    }

    public File getResourceDataPath() {
        return resourceDataPath;
    }

    public void setResourceDataPath(File resourceDataPath) {
        this.resourceDataPath = resourceDataPath;
    }

    public void saveOrgLogo(String id, String fn) throws IOException {
        InputStream in = new FileInputStream(new File(this.tmpPath, fn));
        try {
            IoUtil.copy(in, new FileOutputStream(new File(this.orgPath, id + ".jpg")), false, true);
        } finally {
            in.close();
        }
    }

    public void saveBannerImage(long id, String fn) throws IOException{
        InputStream in = new FileInputStream(new File(this.tmpPath, fn));
        try {
            IoUtil.copy(in,new FileOutputStream(new File(this.bannerPath, id +".jpg")),false,true);
        }finally {
            in.close();
        }
    }

    public void saveDefaultOrgLogo(String id) throws IOException {
        this.initDefaultImage();
        IoUtil.saveStream(new FileOutputStream(new File(this.orgPath, id + ".jpg")), this.dol, true);
    }

    private byte[] zoomImage(byte[] src, int w, int h) throws IOException {
        ByteArrayInputStream in = new ByteArrayInputStream(src);
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        JpgUtil.zoom(in, out, w, h);
        out.flush();
        return out.toByteArray();
    }

    private void initDefaultImage() {
        if (this.dli != null)
            return;
        synchronized (this) {
            if (this.dli != null)
                return;
            try {
                byte[] dd = IoUtil.readStream(new FileInputStream(this.defaultHeadPhoto), true);
                this.dli = this.zoomImage(dd, this.largeHeadPhotoWidth, this.largeHeadPhotoHeight);
                this.dmi = this.zoomImage(dd, this.middleHeadPhotoWidth, this.middleHeadPhotoHeight);
                this.dsi = this.zoomImage(dd, this.smallHeadPhotoWidth, this.smallHeadPhotoHeight);
                this.dol = IoUtil.readStream(new FileInputStream(this.defaultOrgLogo), true);
                this.dsl = IoUtil.readStream(new FileInputStream(this.defaultResourcePhoto), true);
            } catch (IOException e) {
                this.dli = null;
                this.dmi = null;
                this.dsi = null;
                this.dol = null;
                this.dsl = null;
                throw new RuntimeException("init image error", e);
            }
        }
    }

    public boolean hasOrgLogo(String id) {
        String orgPath = this.orgPath+"/"+id+".jpg";
        File file = new File(orgPath);
        if(file.exists()){
            return true;
        }else{
            return false;
        }
    }

    private String createDate() {
        SimpleDateFormat df = new SimpleDateFormat(this.dateFormat);
        String date = df.format(new Date());
        return date;
    }

    private String createResDate() {
        SimpleDateFormat df = new SimpleDateFormat(this.dateFormatRes);
        String date = df.format(new Date());
        return date;
    }

    @Path("/researchResult")
    @Post
    public String researchResult(String fn)throws IOException {
        InputStream in = new FileInputStream(new File(this.tmpPath, fn));
        String savePath = this.researchResultPath + "/" + this.createDate();
        File dateFile = new File(savePath);
        if (!dateFile.exists()) {
            // 创建日期目录
            dateFile.mkdirs();
        }
        String id = StringUtil.buildUUID();
        try {
            IoUtil.copy(in, new FileOutputStream(new File(dateFile, id+ ".jpg")), false, true);
        } finally {
            in.close();
        }
        return this.createDate() + "/" + id + "." + JPG;
    }

    private byte[] readTmpFile(String fn) throws JfwBaseException {
        File file = new File(this.tmpPath, fn);
        if (!file.exists())
            throw new JfwBaseException(90, "resource image not exists");
        try {
            InputStream in = new FileInputStream(file);
            ByteArrayOutputStream out = new ByteArrayOutputStream();
            IoUtil.copy(in, out, true, true);
            return out.toByteArray();
        } catch (IOException e) {
            throw new JfwBaseException(91, "read temp resource image error", e);
        }
    }

    private byte[] resImage(byte[] src, int maxLen) throws IOException {
        ByteArrayInputStream in = new ByteArrayInputStream(src);
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        JpgUtil.scalingZoom(in, out, maxLen);
        out.flush();
        return out.toByteArray();
    }

    public void saveFns(@JdbcConn(true) Connection con, String resourceId, String[] fns, int sort)
            throws SQLException, IOException, JfwBaseException {
        for (int i = 0; i < fns.length; i++) {
            byte[] src = this.readTmpFile(fns[i]);
            src = JpgUtil.read(src);
            byte[] shareResImage = this.resImage(src, this.resImageMaxLen);
            File dateFile = new File(this.resourceDataPath + "/" + this.createResDate());
            if (!dateFile.exists()) {
                // 创建日期目录
                dateFile.mkdirs();
            }
            Image image = new Image();
            String imageId = StringUtil.buildUUID();
            image.setImageId(imageId);
            IoUtil.saveStream(new FileOutputStream(new File(dateFile, imageId + "." + JPG)), src, true);
            IoUtil.saveStream(new FileOutputStream(new File(dateFile, imageId + "_s." + JPG)), shareResImage, true);
            image.setSort(sort+i+1);
            String imageSrc = this.createResDate() + "/" + imageId + "." + JPG;
            image.setImageSrc(imageSrc);
            image.setResourceId(resourceId);
            this.imageDao.insert(con, image);
        }
    }

}
