package com.ekexiu.console.system.service;

import com.ekexiu.console.system.dao.ImageDao;
import com.ekexiu.console.system.dao.ResourceDao;
import com.ekexiu.console.system.po.Image;
import com.ekexiu.console.system.po.Resource;
import com.ekexiu.console.system.pojo.ResourceInfo;
import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.DefaultValue;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.apt.web.annotation.param.PathVar;
import org.jfw.util.PageQueryResult;
import org.jfw.util.exception.JfwBaseException;

import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;

/**
 * Created by TT on 2017/8/9.
 */
@Path("/resource")
public class ResourceService {
    @Autowrie
    private ResourceDao resourceDao;
    @Autowrie
    private KeyWordService keyWordService;
    @Autowrie
    private ImageDao imageDao;
    @Autowrie
    private ImageService imageService;

    public ResourceDao getResourceDao() {
        return resourceDao;
    }

    public void setResourceDao(ResourceDao resourceDao) {
        this.resourceDao = resourceDao;
    }

    public KeyWordService getKeyWordService() {
        return keyWordService;
    }

    public void setKeyWordService(KeyWordService keyWordService) {
        this.keyWordService = keyWordService;
    }

    public ImageDao getImageDao() {
        return imageDao;
    }

    public void setImageDao(ImageDao imageDao) {
        this.imageDao = imageDao;
    }

    public ImageService getImageService() {
        return imageService;
    }

    public void setImageService(ImageService imageService) {
        this.imageService = imageService;
    }

    @Path("/pq")
    @Get
    public PageQueryResult<ResourceInfo> pageQueryResult(@JdbcConn Connection con, @Nullable String resourceName, @Nullable String name, int orderBy, @Nullable String bt, @Nullable String et, @DefaultValue("1") int pageNo, @DefaultValue("10") int pageSize)throws SQLException {
        if (orderBy == 1) {
            return this.resourceDao.queryByTime(con, resourceName == null ? null : "%" + resourceName + "%",  name == null ? "%" : "%" + name + "%", bt == null ? null : bt + "000000", et == null ? null : et + "235959", pageSize, pageNo);
        }
        if (orderBy == 2) {
            return this.resourceDao.queryByPV(con, resourceName == null ? null : "%" + resourceName + "%",  name == null ? "%" : "%" + name + "%", bt == null ? null : bt + "000000", et == null ? null : et + "235959", pageSize, pageNo);
        }
        return null;
    }

    @Post
    @Path("/deleteResource")
    public void deleteResource(@JdbcConn(true) Connection con, String resourceId) throws SQLException {
        //修改文章状态为删除
        this.resourceDao.updateStatus(con, resourceId, "3");
    }

    @Post
    @Path("/sortNum")
    public void updateSortNum(@JdbcConn(true) Connection con,String resourceId,long sortNum) throws SQLException{
        this.resourceDao.updateSortNum(con, resourceId, sortNum);
    }

    @Get
    @Path("/id/{id}")
    public Resource query(@JdbcConn Connection con, @PathVar String id) throws SQLException {
        return this.resourceDao.query(con, id);
    }

    @Get
    @Path("/resourceInfo")
    public ResourceInfo queryInfo(@JdbcConn Connection con, String id)throws SQLException {
        return this.resourceDao.queryResInfo(con, id);
    }

    @Post
    @Path("/subject")
    public void updateSubject(@JdbcConn(true) Connection con, String resourceId, @Nullable String subject) throws SQLException {
        this.resourceDao.updateSubject(con, resourceId, subject);
        this.keyWordService.refreshResource(con, resourceId, subject);
    }

    @Get
    @Path("/queryOne")
    public Resource queryOne(@JdbcConn Connection con, String resourceId)throws SQLException {
        Resource resource = this.resourceDao.query(con, resourceId);
        if (resource != null) {
            resource.setImages(this.imageDao.queryRes(con,resourceId));
        }
        return resource;
    }

    @Post
    @Path("/save")
    public String save(@JdbcConn(true) Connection con, Resource resource, @Nullable String[] fns, @Nullable String[] imageIds)throws JfwBaseException,SQLException,IOException {
        if (resource.getResourceId().trim().length() == 32) {
            int imageLength = 0;
            if (imageIds != null) {
                imageLength = imageIds.length;
                List<Image> images = this.imageDao.queryRes(con, resource.getResourceId());
                this.imageDao.updateAllSort(con, resource.getResourceId(), 0);
                for (int i=0;i<imageLength;i++) {
                    for (Image image : images) {
                        if (image.getImageId().equals(imageIds[i])) {
                            this.imageDao.updateSort(con, image.getImageId(), i + 1);
                        }
                    }
                }
                this.imageDao.deleteBySort(con, resource.getResourceId(), 0);
            }
            if (fns != null) {
                if (imageIds == null) {
                    this.imageDao.deleteRes(con, resource.getResourceId());
                }
                this.imageService.saveFns(con,resource.getResourceId(),fns,imageLength);
            }
                this.resourceDao.update(con, resource);
            this.keyWordService.refreshResource(con,resource.getResourceId(),resource.getSubject());
            return resource.getResourceId();
        }else {
            throw new JfwBaseException(-1, "bad parameter:resourceId");
        }
    }
}
